﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TaleWorlds.Core;
using TaleWorlds.Library;
using TaleWorlds.SaveSystem;
using TaleWorlds.InputSystem;
using TaleWorlds.Localization;
using TaleWorlds.MountAndBlade;
using TaleWorlds.CampaignSystem;

namespace CompanionRespec
{
    // Thanks Ster!

    public static class ConsoleCommand
    {
        [CommandLineFunctionality.CommandLineArgumentFunction("respec_allcompanions", "campaign")]
        public static string Respec_Companions(List<string> strings)
        {
            if (Campaign.Current == null)
            {
                return "Campaign is null";
            }

            Hero hero = Hero.MainHero;

            if (hero.PartyBelongedTo != null && hero.Clan != null)
            {
                if (hero.Clan.Companions.Count == 0)
                {
                    return "You have no companions";
                }

                foreach (Hero companion in hero.Clan.Companions)
                {
                    if (companion.PartyBelongedTo == hero.PartyBelongedTo)
                    {
                        CompanionRespecBehavior i = new CompanionRespecBehavior();
                        i.RespecHero(companion);
                    }
                }
                if (hero.Clan.Companions.Count > hero.CompanionsInParty.Count())
                {
                    int excluded = hero.Clan.Companions.Count - hero.CompanionsInParty.Count();
                    return $"Done. {excluded} companion(s) were excluded (not in party)";
                }
                return "Done";

            }

            return "Party or Clan is null. What have you done?!";
        }

        [CommandLineFunctionality.CommandLineArgumentFunction("respec_mainhero", "campaign")]
        public static string Respec_MainHero(List<string> strings)
        {
            if (Campaign.Current == null)
            {
                return "Campaign is null";
            }

            Hero hero = Hero.MainHero;
            CompanionRespecBehavior respecBehavior = new CompanionRespecBehavior();
            respecBehavior.RespecHero(hero);

            return "Done";
        }


        [CommandLineFunctionality.CommandLineArgumentFunction("respec_companion", "campaign")]
        public static string Respec_1Hero(List<string> strings)
        {

            if (Campaign.Current == null)
            {
                return "Campaign is null";
            }

            if (CampaignCheats.CheckParameters(strings, 0) || CampaignCheats.CheckHelp(strings))
            {
                return "Format is \"campaign.respec_companion [Hero Name]\"";
            }
            string heroName = string.Join(" ", strings.ToArray()).Trim(new char[]
            {
                '"'
            });

            if (Hero.MainHero.Name.ToString().ToLower().Contains(heroName.ToLower()))
            {
                return "Hero is MainHero - use \"campaign.respec_mainhero\"";
            }

            // Hero hero = Hero.MainHero.CompanionsInParty.FirstOrDefault((Hero h) => h.Name.ToString().ToLower().Contains(heroName.ToLower()));

            Hero hero = Hero.MainHero.Clan.Companions.FirstOrDefault((Hero h) => h.Name.ToString().ToLower().Contains(heroName.ToLower()));

            if (hero == null)
            {
                return "Hero not found";
            }

            if (!Hero.MainHero.CompanionsInParty.Contains(hero))
            {
                return "Companion must be in current party";
            }

            CompanionRespecBehavior i = new CompanionRespecBehavior();
            i.RespecHero(hero);

            return "Done";
        }

    }
}
